clear
close all
clc
warning off

tf=15; % stop time
ts=0.1; % sample time
t=0:ts:tf; % vector time

Q=length(t); % samples

%%%%%%%%%%%%%%%%%%% Initial conditions %%%%%%%%%%%%%%%%%%%%%

hx = zeros(1,Q+1);  % initial position (x axis) in meters (m)
hy = zeros(1,Q+1);  % initial position (y axis) in meters (m)
phi = zeros(1,Q+1); % initial orientation in radians (rad)

hxReal = zeros(1,Q+1);  % initial real position (x axis) in meters (m)
hyReal = zeros(1,Q+1);  % initial real center position (y axis) in meters (m)
phiReal = zeros(1,Q+1); % initial real orientation in radians (rad)

uRef=0.2*sin(0.1*t); % reference linear velocity (m/s)
wRef=1*ones(1,Q); % reference angular velocity (rad/s)

uMeas=zeros(1,Q);  % measurement linear velocity (m/s)
wMeas=zeros(1,Q);  % measurement angular velocity (rad/s)

tiempo=zeros(1,Q); 
%%

COM='COM5'; % Port Arduino

delete(instrfind({'Port'},{COM}));

serialPort=serial(COM);
set(serialPort,'Baudrate',9600); 
set(serialPort,'StopBits',1); 
set(serialPort,'DataBits',8); 
set(serialPort,'Parity','none');
set(serialPort,'Terminator','CR/LF');
set(serialPort,'OutputBufferSize',30); 
set(serialPort,'InputBufferSize' ,30); 
set(serialPort,'Timeout',5);




fopen(serialPort); % open port

pause(3); % delay 3 seconds


for k=1:Q
    
    tic

    trama=[num2str(round(uRef(k),2)) ',' num2str(round(wRef(k),2))]; % String data 
    
    fprintf(serialPort,'%s\n',trama,'sync'); % send data (String)

    v1 = fscanf(serialPort, '%f\n'); % receive data (u)
    v2 = fscanf(serialPort, '%f\n'); % receive data (w)

    uMeas(k)=v1; 
    wMeas(k)=v2;

%%%%%%%%%%%%%%%%%% Simulated robot (Kinematic Model) %%%%%%%%%%%%%%%%%%%%%%

    phi(k+1)=phi(k)+ts*wRef(k); 
    hxp = uRef(k)*cos(phi(k+1));
    hyp = uRef(k)*sin(phi(k+1));

    % numerical integral (Euler Method)
    hx(k+1)=hx(k)+ts*hxp; 
    hy(k+1)=hy(k)+ts*hyp; 

%%%%%%%%%%%%%%%%%% Real robot (Kinematic Model) %%%%%%%%%%%%%%%%%%%%%%

    phiReal(k+1)=phiReal(k)+ts*wMeas(k);   % real orientation in radians
    hxpReal = uMeas(k)*cos(phiReal(k+1)); % real velocity in meters/seconds
    hypReal = uMeas(k)*sin(phiReal(k+1)); % real velocity in meters/seconds

    % b) numerical integral (Euler Method)
    hxReal(k+1)=hxReal(k)+ts*hxpReal;  % real center position (x axis) in meters (m)
    hyReal(k+1)=hyReal(k)+ts*hypReal;  % real center position (y axis) in meters (m)

    tiempo(k)=toc;

    while toc<ts
    end
    
end

fprintf(serialPort,'%s\n','0,0','sync');

%%
figure
subplot(211)
plot(t,uMeas,'b','LineWidth',2),hold on, plot(t,uRef,'r','LineWidth',2),legend('Measurement','Reference'),title('Linear Velocity'),xlabel('Time (s)'),ylabel('m/s'), grid on;
subplot(212)
plot(t,wMeas,'b','LineWidth',2),hold on, plot(t,wRef,'r','LineWidth',2),legend('Measurement','Reference'),title('Angular Velocity'),xlabel('Time (s)'),ylabel('rad/s'), grid on;
figure
plot(tiempo,'LineWidth',2),title('Time'),xlabel('Samples'),ylabel('Time (s)'), grid on;
figure
plot(hx(1),hy(1),'gO'),hold on,plot(hx(end),hy(end),'rO'),hold on,plot(hx,hy,'b','LineWidth',2),hold on,plot(hxReal,hyReal,'r','LineWidth',2),title('Trajectory'),xlabel('Time (s)'),ylabel('m'),grid on;
axis([-2 2 -1 1]);
%%
%%%%%%%%%%%%%%%%%%%%% Stroboscopic movement of the ROBOT %%%%%%%%%%%%%%%%%%%
scene=figure;  % new figure
tam=get(0,'ScreenSize');
set(scene,'position',[tam(1) tam(2) tam(3) tam(4)]); % position and size figure in the screen
axis equal; % Set axis aspect ratios
axis([-2 2 -2 2 0 1]); % Set axis limits 
view([57 38]); % orientation figure
grid on; % Display axes grid lines

MobileRobot; % Parameters of robot 
M1=MobilePlot(hxReal(1),hyReal(1),phiReal(1),0.5); % Plot robot in initial position x1,y1 and phi orientation
hold on
xlabel('x(m)'); ylabel('y(m)'); zlabel('z(m)'); % Label axis
camlight right % scene light

M2=plot3(hxReal(1),hyReal(1),0.13,'gO','LineWidth',2); % Initial point.
M3=plot3(hxReal(end),hyReal(end),0.13,'rO','LineWidth',2);% Final point.
M4=plot3(hx(1),hy(1),zeros(1),'b','LineWidth',2); % Initial trajectory.

step=1; % step for simulation

for i=1:step:k % Loop emulation

     delete (M1) % delete trajectory
     delete (M4) % delete robot
     M1=MobilePlot(hxReal(i),hyReal(i),phiReal(i),0.5); hold on; % Plot of robot again
     M4=plot3(hxReal(1:i),hyReal(1:i),zeros(1,i),'b','LineWidth',2); % plot trajectory.
     
     pause(ts)
     % sample time
end

%%
fprintf(serialPort,'%s\n','0,0','sync');
fclose(serialPort); 
delete(serialPort);
clear PS;
